// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/FastJets.hh"
#include "Rivet/Projections/HeavyHadrons.hh"
#include "Rivet/Projections/IdentifiedFinalState.hh"

namespace Rivet {


  class D0_2000_I499943 : public Analysis {
  public:

    /// Constructor
    D0_2000_I499943()
      : Analysis("D0_2000_I499943")
    {    }
    
    
  public:
    
    /// @name Analysis methods
    //@{
    
    /// Book histograms and initialise projections before the run
    void init() {
      
      /// @todo Initialise and register projections here
      FinalState fs;
      // projection to find the muons
      IdentifiedFinalState muons(-0.8, 0.8,  4.0*GeV);
      muons.acceptIdPair(PID::MUON);
      addProjection(muons, "muons");
      
      FastJets jetproj(fs, FastJets::D0ILCONE, 0.7);
      jetproj.useInvisibles();
      addProjection(jetproj, "Jets");
      
      /// @todo Book histograms here, e.g.:
      _h_pt_leading_mu = bookHisto1D(1, 1, 1);
      _h_dphi_mumu = bookHisto1D(3, 1, 1);
      
    }
    
    
    /// Perform the per-event analysis
    void analyze(const Event& event) {
      const double weight = event.weight();

      const Jets& jets = applyProjection<FastJets>(event,"Jets").jetsByPt(12*GeV);
      if(jets.size()<2 ) vetoEvent;

      const Particles& muons =applyProjection<IdentifiedFinalState>(event, "muons").particlesByPt();
      if(muons.size()<2) vetoEvent;




      //muons selection
      Particles cand_mu;
      foreach ( const Particle& mu, applyProjection<IdentifiedFinalState>(event, "muons").particlesByPt() ) {

	//removing muons in "bad" region 80<phi<110 degrees
	if( mu.phi()>1.4 && mu.phi()<1.92) continue; 

	//removing muons not associated with jets
	bool match = false;
	foreach ( const Jet& jet, jets ) {
	  //	  cout<<"mu phi="<<mu.phi()<<" eta="<<mu.eta()<<endl;
	  //	  cout<<"jet phi="<<jet.phi()<<" eta="<<jet.eta()<<endl;
	  //	  cout<<"deltaR="<<deltaR(mu.momentum(),jet.momentum())<<endl;
          if ( deltaR(mu.momentum(),jet.momentum()) < 0.8 ) match = true;
        }
	if(match) cand_mu.push_back(mu);
      }

      if(cand_mu.size()<2 ) vetoEvent;

      //removing muon pairs closer than 165 deg in space angle (cosmic veto)
      //      double dR_mumu = deltaR(cand_mu[0].momentum(), cand_mu[1].momentum());
      //      if(dR_mumu<165*degree) vetoEvent;


      //selecting muon pairs with 6<m<35 GeV
      double m_mumu = (cand_mu[0].momentum()+cand_mu[1].momentum()).mass();
      if(m_mumu<6*GeV || m_mumu>35*GeV) vetoEvent;

      double pTmu = cand_mu[0].pt();
      double dphi_mumu = deltaPhi(cand_mu[0].phi(), cand_mu[1].phi());
      dphi_mumu*=180/3.1415;
      //      cout<<"mu0_pt="<<cand_mu[0].pt()<<"  mu1_pt="<<cand_mu[1].pt()<<" dphi="<<dphi_mumu<<endl;

      _h_pt_leading_mu->fill(pTmu/GeV, weight);
      _h_dphi_mumu->fill(dphi_mumu, weight);

    }


    /// Normalise histograms etc., after the run
    void finalize() {

      scale(_h_pt_leading_mu, crossSection()/sumOfWeights()/nanobarn); // norm to cross section
      scale(_h_dphi_mumu, crossSection()/sumOfWeights()/nanobarn); // norm to cross section

    }

    //@}


  private:

    // Data members like post-cuts event weight counters go here


  private:

    /// @name Histograms
    //@{
    Histo1DPtr _h_pt_leading_mu;
    Histo1DPtr _h_dphi_mumu;
    //@}


  };



  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(D0_2000_I499943);

}
